'use strict'

const uid = require('uid')
const C = require('were-constants')
const createMessageRegexp = require('lib/create-message-regexp')
const log = require('lib/log')


const HydroSocket = function (socket) {

  var self = this

  this.subscriptions = []
  this.socket = socket
  this.dataHandler = function (message) {

    message.toString('utf8')
      .split(C.MESSAGE_SEPERATOR)
      .slice(0, -1)
      .forEach(function (message) {

        console.log(message)
        self.subscriptions
          .filter(function (subscription) {
            return message.match(subscription.regexp)
          })
          .forEach(function (subscription) {
            (function (topic, action, id, name, data) {
              subscription.callback(
                topic,
                action,
                id,
                name,
                JSON.parse(data)
              )
            })
            .apply(this, message.split(C.MESSAGE_PART_SEPERATOR))
          })
      })
  }

  this.socket.on('data', this.dataHandler.bind(this))
  this.socket.once('close', this.discard.bind(this))
}

HydroSocket.prototype.send = function (options) {
  log.tag('SEND')
    .debug(options)

  var socket = this.socket

  return new Promise(function (resolve, reject) {

    if (!options.hasOwnProperty('id'))
      options.id = uid(10);

    socket.write([
        options.topic,
        options.action,
        options.id,
        options.name || '',
        JSON.stringify((typeof options.data === 'undefined') ? {} :
          options.data)
      ]
      .join(C.MESSAGE_PART_SEPERATOR) + C.MESSAGE_SEPERATOR)

  })
  .catch(console.log)
}

HydroSocket.prototype.subscribe = function (options, callback) {
  log.tag('SUBSCRIBE')
    .debug(options)

  this.subscriptions.push({
    regexp: createMessageRegexp(options),
    callback: callback
  })
}

HydroSocket.prototype.unsubscribe = function (options) {
  log.tag('UNSUBSCRIBE')
    .debug(options)

  var regexp = createMessageRegexp(options)

  this.subscriptions = this.subscriptions.filter(function (subscription) {
    return regexp !== subscription.regexp
  })
}

HydroSocket.prototype.discard = function () {

  this.socket.removeListener('data', this.dataHandler)
}

module.exports = HydroSocket
